@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 to 2014 by Jonathan Duddington
  * email: jonsd@users.sourceforge.net
- * Copyright (C) 2015-2016 Reece H. Dunn
+ * Copyright (C) 2015-2017 Reece H. Dunn
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 
 #include <espeak-ng/espeak_ng.h>
 #include <espeak-ng/speak_lib.h>
+#include <espeak-ng/encoding.h>
 
 #include "error.h"
 #include "speech.h"
@@ -51,8 +52,15 @@ static int text_mode = 0;
 static int debug_flag = 0;
 static int error_need_dictionary = 0;
 
-static int hash_counts[N_HASH_DICT];
+// A hash chain is a linked-list of hash chain entry objects:
+//     struct hash_chain_entry {
+//         hash_chain_entry *next_entry;
+//         // dict_line output from compile_line:
+//         uint8_t length;
+//         char contents[length];
+//     };
 static char *hash_chains[N_HASH_DICT];
+
 static char letterGroupsDefined[N_LETTER_GROUPS];
 
 MNEM_TAB mnem_rules[] = {
@@ -173,17 +181,6 @@ int isspace2(unsigned int c)
 	return 1;
 }
 
-// Lookup a mnemonic string in a table, return its name
-const char *LookupMnemName(MNEM_TAB *table, const int value)
-{
-	while (table->mnem != NULL) {
-		if (table->value == value)
-			return table->mnem;
-		table++;
-	}
-	return ""; // not found
-}
-
 void print_dictionary_flags(unsigned int *flags, char *buf, int buf_len)
 {
 	int stress;
@@ -372,15 +369,26 @@ char *DecodeRule(const char *group_chars, int group_length, char *rule, int cont
 	return output;
 }
 
-static int compile_line(char *linebuf, char *dict_line, int *hash)
+typedef enum
+{
+	LINE_PARSER_WORD = 0,
+	LINE_PARSER_END_OF_WORD = 1,
+	LINE_PARSER_MULTIPLE_WORDS = 2,
+	LINE_PARSER_END_OF_WORDS = 3,
+	LINE_PARSER_PRONUNCIATION = 4,
+	LINE_PARSER_END_OF_PRONUNCIATION = 5,
+} LINE_PARSER_STATES;
+
+static int compile_line(char *linebuf, char *dict_line, int n_dict_line, int *hash)
 {
 	// Compile a line in the language_list file
 	unsigned char c;
 	char *p;
 	char *word;
 	char *phonetic;
+	char *phonetic_end;
 	unsigned int ix;
-	int step;
+	LINE_PARSER_STATES step;
 	unsigned int n_flag_codes = 0;
 	int flagnum;
 	int flag_offset;
@@ -408,10 +416,10 @@ static int compile_line(char *linebuf, char *dict_line, int *hash)
 
 	p = linebuf;
 
-	step = 0;
+	step = LINE_PARSER_WORD;
 
-	c = 0;
-	while (c != '\n') {
+	c = *p;
+	while (c != '\n' && c != '\0') {
 		c = *p;
 
 		if ((c == '?') && (step == 0)) {
@@ -465,17 +473,17 @@ static int compile_line(char *linebuf, char *dict_line, int *hash)
 
 		switch (step)
 		{
-		case 0:
+		case LINE_PARSER_WORD:
 			if (c == '(') {
 				multiple_words = 1;
 				word = p+1;
-				step = 1;
+				step = LINE_PARSER_END_OF_WORD;
 			} else if (!isspace2(c)) {
 				word = p;
-				step = 1;
+				step = LINE_PARSER_END_OF_WORD;
 			}
 			break;
-		case 1:
+		case LINE_PARSER_END_OF_WORD:
 			if ((c == '-') && multiple_words) {
 				if (IsDigit09(word[0]))
 					multiple_numeric_hyphen = 1;
@@ -487,43 +495,48 @@ static int compile_line(char *linebuf, char *dict_line, int *hash)
 
 				if (multiple_words) {
 					multiple_string = multiple_string_end = p+1;
-					step = 2;
+					step = LINE_PARSER_MULTIPLE_WORDS;
 				} else
-					step = 3;
+					step = LINE_PARSER_END_OF_WORDS;
 			} else if (c == ')') {
 				if (multiple_words) {
 					p[0] = 0;
 					multiple_words = 0;
-					step = 3;
+					step = LINE_PARSER_END_OF_WORDS;
 				} else if (word[0] != '_') {
 					fprintf(f_log, "%5d: Missing '('\n", linenum);
 					error_count++;
-					step = 3;
+					step = LINE_PARSER_END_OF_WORDS;
 				}
 			}
 			break;
-		case 2:
+		case LINE_PARSER_MULTIPLE_WORDS:
 			if (isspace2(c))
 				multiple_words++;
 			else if (c == ')') {
 				p[0] = ' '; // terminate extra string
 				multiple_string_end = p+1;
-				step = 3;
+				step = LINE_PARSER_END_OF_WORDS;
 			}
 			break;
-		case 3:
+		case LINE_PARSER_END_OF_WORDS:
 			if (!isspace2(c)) {
 				phonetic = p;
-				step = 4;
+				step = LINE_PARSER_PRONUNCIATION;
 			}
 			break;
-		case 4:
+		case LINE_PARSER_PRONUNCIATION:
 			if (isspace2(c)) {
+				phonetic_end = p;
 				p[0] = 0; // terminate phonetic
-				step = 5;
+				step = LINE_PARSER_END_OF_PRONUNCIATION;
 			}
 			break;
-		case 5:
+		case LINE_PARSER_END_OF_PRONUNCIATION:
+			if (!isspace2(c)) {
+				*phonetic_end = ' ';
+				step = LINE_PARSER_PRONUNCIATION;
+			}
 			break;
 		}
 		p++;
@@ -586,7 +599,7 @@ static int compile_line(char *linebuf, char *dict_line, int *hash)
 			ix = utf8_in(&c2, p);
 			if (c2 == 0)
 				break;
-			if (iswupper2(c2))
+			if (iswupper(c2))
 				utf8_out(towlower2(c2), p);
 			else
 				all_upper_case = 0;
@@ -615,7 +628,15 @@ static int compile_line(char *linebuf, char *dict_line, int *hash)
 		length = len_word + 2;
 	} else {
 		length = len_word + len_phonetic + 3;
-		strcpy(&dict_line[(len_word)+2], encoded_ph);
+		if (length < n_dict_line) {
+			strcpy(&dict_line[(len_word)+2], encoded_ph);
+		} else {
+			fprintf(f_log, "%5d: Dictionary line length would overflow the data buffer: %d\n", linenum, length);
+			error_count++;
+			// no phonemes specified. set bit 7
+			dict_line[1] |= 0x80;
+			length = len_word + 2;
+		}
 	}
 
 	for (ix = 0; ix < n_flag_codes; ix++)
@@ -635,7 +656,7 @@ static int compile_line(char *linebuf, char *dict_line, int *hash)
 			length += ix;
 		}
 	}
-	dict_line[0] = length;
+	*((uint8_t *)dict_line) = (uint8_t)length;
 
 	return length;
 }
@@ -655,7 +676,6 @@ static void compile_dictlist_start(void)
 			p = p2;
 		}
 		hash_chains[ix] = NULL;
-		hash_counts[ix] = 0;
 	}
 }
 
@@ -668,10 +688,9 @@ static void compile_dictlist_end(FILE *f_out)
 
 	for (hash = 0; hash < N_HASH_DICT; hash++) {
 		p = hash_chains[hash];
-		hash_counts[hash] = (int)ftell(f_out);
 
 		while (p != NULL) {
-			length = *(p+sizeof(char *));
+			length = *(uint8_t *)(p+sizeof(char *));
 			fwrite(p+sizeof(char *), length, 1, f_out);
 			memcpy(&p, p, sizeof(char *));
 		}
@@ -688,7 +707,7 @@ static int compile_dictlist_file(const char *path, const char *filename)
 	FILE *f_in;
 	char buf[200];
 	char fname[sizeof(path_home)+45];
-	char dict_line[128];
+	char dict_line[256]; // length is uint8_t, so an entry can't take up more than 256 bytes
 
 	text_mode = 0;
 
@@ -708,11 +727,9 @@ static int compile_dictlist_file(const char *path, const char *filename)
 	while (fgets(buf, sizeof(buf), f_in) != NULL) {
 		linenum++;
 
-		length = compile_line(buf, dict_line, &hash);
+		length = compile_line(buf, dict_line, sizeof(dict_line), &hash);
 		if (length == 0)  continue; // blank line
 
-		hash_counts[hash]++;
-
 		p = (char *)malloc(length+sizeof(char *));
 		if (p == NULL) {
 			if (f_log != NULL) {
@@ -724,6 +741,7 @@ static int compile_dictlist_file(const char *path, const char *filename)
 
 		memcpy(p, &hash_chains[hash], sizeof(char *));
 		hash_chains[hash] = p;
+		// NOTE: dict_line[0] is the entry length (0-255)
 		memcpy(p+sizeof(char *), dict_line, length);
 		count++;
 	}
@@ -1564,6 +1582,7 @@ ESPEAK_NG_API espeak_ng_STATUS espeak_ng_CompileDictionary(const char *dsource,
 		compile_dictlist_file(path, "listx");
 		compile_dictlist_file(path, "list");
 	}
+	compile_dictlist_file(path, "emoji");
 	compile_dictlist_file(path, "extra");
 
 	compile_dictlist_end(f_out);
