@@ -1,5 +1,5 @@
 
-#define TEST_FP
+//#define TEST_FP
 
 /* Copyright (C) 1989-1991 Apple Computer, Inc.
  *
@@ -978,3 +978,4 @@ extended(1.99111) --> IEEE(1.99111) [3FFF FEDCBA98 76543210]
 */
 
 #endif TEST_FP
+
