@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 to 2015 by Jonathan Duddington
  * email: jonsd@users.sourceforge.net
- * Copyright (C) 2015-2016 Reece H. Dunn
+ * Copyright (C) 2015-2017 Reece H. Dunn
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@
 
 #include <espeak-ng/espeak_ng.h>
 #include <espeak-ng/speak_lib.h>
+#include <espeak-ng/encoding.h>
+#include <ucd/ucd.h>
 
 #include "error.h"
 #include "speech.h"
@@ -49,13 +51,10 @@ static int namedata_ix = 0;
 static int n_namedata = 0;
 char *namedata = NULL;
 
-static FILE *f_input = NULL;
 static int ungot_char2 = 0;
-unsigned char *p_textinput;
-wchar_t *p_wchar_input;
+espeak_ng_TEXT_DECODER *p_decoder = NULL;
 static int ungot_char;
 static const char *ungot_word = NULL;
-static int end_of_input;
 
 static int ignore_text = 0; // set during <sub> ... </sub>  to ignore text which has been replaced by an alias
 static int audio_text = 0; // set during <audio> ... </audio>
@@ -65,117 +64,6 @@ static int sayas_mode;
 static int sayas_start;
 static int ssml_ignore_l_angle = 0;
 
-// punctuations symbols that can end a clause
-static const unsigned short punct_chars[] = {
-	',', '.', '?', '!', ':', ';',
-
-	0x00a1, // inverted exclamation
-	0x00bf, // inverted question
-	0x2013, // en-dash
-	0x2014, // em-dash
-	0x2026, // elipsis
-
-	0x037e, // Greek question mark (looks like semicolon)
-	0x0387, // Greek semicolon, ano teleia
-	0x0964, // Devanagari Danda (fullstop)
-
-	0x0589, // Armenian period
-	0x055d, // Armenian comma
-	0x055c, // Armenian exclamation
-	0x055e, // Armenian question
-	0x055b, // Armenian emphasis mark
-
-	0x060c, // Arabic ,
-	0x061b, // Arabic ;
-	0x061f, // Arabic ?
-	0x06d4, // Arabic .
-
-	0x0df4, // Singhalese Kunddaliya
-	0x0f0d, // Tibet Shad
-	0x0f0e,
-
-	0x1362, // Ethiopic period
-	0x1363,
-	0x1364,
-	0x1365,
-	0x1366,
-	0x1367,
-	0x1368,
-	0x10fb, // Georgian paragraph
-
-	0x3001, // ideograph comma
-	0x3002, // ideograph period
-
-	0xff01, // fullwidth exclamation
-	0xff0c, // fullwidth comma
-	0xff0e, // fullwidth period
-	0xff1a, // fullwidth colon
-	0xff1b, // fullwidth semicolon
-	0xff1f, // fullwidth question mark
-
-	0
-};
-
-// indexed by (entry num. in punct_chars) + 1
-// bits 0-7 pause x 10mS, bits 12-14 intonation type, bit 15 don't need following space or bracket
-static const unsigned int punct_attributes[] = {
-	0,
-
-	CLAUSE_COMMA,
-	CLAUSE_PERIOD,
-	CLAUSE_QUESTION,
-	CLAUSE_EXCLAMATION,
-	CLAUSE_COLON,
-	CLAUSE_SEMICOLON,
-
-	CLAUSE_SEMICOLON | 0x8000,  // inverted exclamation
-	CLAUSE_SEMICOLON | 0x8000,  // inverted question
-	CLAUSE_SEMICOLON,  // en-dash
-	CLAUSE_SEMICOLON,  // em-dash
-	CLAUSE_SEMICOLON | PUNCT_SAY_NAME | 0x8000,  // elipsis
-
-	CLAUSE_QUESTION,  // Greek question mark
-	CLAUSE_SEMICOLON,  // Greek semicolon
-	CLAUSE_PERIOD | 0x8000,  // Devanagari Danda (fullstop)
-
-	CLAUSE_PERIOD | 0x8000,  // Armenian period
-	CLAUSE_COMMA,  // Armenian comma
-	CLAUSE_EXCLAMATION | PUNCT_IN_WORD,  // Armenian exclamation
-	CLAUSE_QUESTION | PUNCT_IN_WORD,  // Armenian question
-	CLAUSE_PERIOD | PUNCT_IN_WORD,  // Armenian emphasis mark
-
-	CLAUSE_COMMA,  // Arabic ,
-	CLAUSE_SEMICOLON,  // Arabic ;
-	CLAUSE_QUESTION,  // Arabic question mark
-	CLAUSE_PERIOD,  // Arabic full stop
-
-	CLAUSE_PERIOD+0x8000,  // Singhalese period
-	CLAUSE_PERIOD+0x8000,  // Tibet period
-	CLAUSE_PARAGRAPH,
-
-	CLAUSE_PERIOD,  // Ethiopic period
-	CLAUSE_COMMA,  // Ethiopic comma
-	CLAUSE_SEMICOLON,  // Ethiopic semicolon
-	CLAUSE_COLON,  // Ethiopic colon
-	CLAUSE_COLON,  // Ethiopic preface colon
-	CLAUSE_QUESTION,  // Ethiopic question mark
-	CLAUSE_PARAGRAPH,  // Ethiopic paragraph
-	CLAUSE_PARAGRAPH,  // Georgian paragraph
-
-	CLAUSE_COMMA+0x8000,  // ideograph comma
-	CLAUSE_PERIOD+0x8000,  // ideograph period
-
-	CLAUSE_EXCLAMATION+0x8000,  // fullwidth
-	CLAUSE_COMMA+0x8000,
-	CLAUSE_PERIOD+0x8000,
-	CLAUSE_COLON+0x8000,
-	CLAUSE_SEMICOLON+0x8000,
-	CLAUSE_QUESTION+0x8000,
-
-	CLAUSE_SEMICOLON,  // spare
-	0
-};
-
 // stack for language and voice properties
 // frame 0 is for the defaults, before any ssml tags.
 typedef struct {
@@ -202,6 +90,55 @@ PARAM_STACK param_stack[N_PARAM_STACK];
 static int speech_parameters[N_SPEECH_PARAM]; // current values, from param_stack
 int saved_parameters[N_SPEECH_PARAM]; // Parameters saved on synthesis start
 
+#define ESPEAKNG_CLAUSE_TYPE_PROPERTY_MASK 0xFFF0000000000000ull
+
+int clause_type_from_codepoint(uint32_t c)
+{
+	ucd_category cat = ucd_lookup_category(c);
+	ucd_property props = ucd_properties(c, cat);
+
+	switch (props & ESPEAKNG_CLAUSE_TYPE_PROPERTY_MASK)
+	{
+	case ESPEAKNG_PROPERTY_FULL_STOP:
+		return CLAUSE_PERIOD;
+	case ESPEAKNG_PROPERTY_FULL_STOP | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER:
+		return CLAUSE_PERIOD | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_QUESTION_MARK:
+		return CLAUSE_QUESTION;
+	case ESPEAKNG_PROPERTY_QUESTION_MARK | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER:
+		return CLAUSE_QUESTION | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_QUESTION_MARK | ESPEAKNG_PROPERTY_PUNCTUATION_IN_WORD:
+		return CLAUSE_QUESTION | CLAUSE_PUNCTUATION_IN_WORD;
+	case ESPEAKNG_PROPERTY_EXCLAMATION_MARK:
+		return CLAUSE_EXCLAMATION;
+	case ESPEAKNG_PROPERTY_EXCLAMATION_MARK | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER:
+		return CLAUSE_EXCLAMATION | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_EXCLAMATION_MARK | ESPEAKNG_PROPERTY_PUNCTUATION_IN_WORD:
+		return CLAUSE_EXCLAMATION | CLAUSE_PUNCTUATION_IN_WORD;
+	case ESPEAKNG_PROPERTY_COMMA:
+		return CLAUSE_COMMA;
+	case ESPEAKNG_PROPERTY_COMMA | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER:
+		return CLAUSE_COMMA | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_COLON:
+		return CLAUSE_COLON;
+	case ESPEAKNG_PROPERTY_COLON | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER:
+		return CLAUSE_COLON | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_SEMI_COLON:
+	case ESPEAKNG_PROPERTY_EXTENDED_DASH:
+		return CLAUSE_SEMICOLON;
+	case ESPEAKNG_PROPERTY_SEMI_COLON | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER:
+	case ESPEAKNG_PROPERTY_QUESTION_MARK | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER | ESPEAKNG_PROPERTY_INVERTED_TERMINAL_PUNCTUATION:
+	case ESPEAKNG_PROPERTY_EXCLAMATION_MARK | ESPEAKNG_PROPERTY_OPTIONAL_SPACE_AFTER | ESPEAKNG_PROPERTY_INVERTED_TERMINAL_PUNCTUATION:
+		return CLAUSE_SEMICOLON | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_ELLIPSIS:
+		return CLAUSE_SEMICOLON | CLAUSE_SPEAK_PUNCTUATION_NAME | CLAUSE_OPTIONAL_SPACE_AFTER;
+	case ESPEAKNG_PROPERTY_PARAGRAPH_SEPARATOR:
+		return CLAUSE_PARAGRAPH;
+	}
+
+	return CLAUSE_NONE;
+}
+
 const int param_defaults[N_SPEECH_PARAM] = {
 	0,   // silence (internal use)
 	175, // rate wpm
@@ -220,293 +157,13 @@ const int param_defaults[N_SPEECH_PARAM] = {
 	0,   // voice type
 };
 
-// additional Latin characters beyond the ascii character set
-#define MAX_WALPHA  0x24f
-// indexed by character - 0x80
-// 0=not alphabetic, 0xff=lower case, 0xfe=no case, 0xfd=use wchar_tolower
-//   other=value to add to upper case to convert to lower case
-static unsigned char walpha_tab[MAX_WALPHA-0x7f] = {
-	   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, // 080
-	   0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, // 090
-	   0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 0xfe,    0,    0,    0,    0,    0, // 0a0
-	   0,    0,    0,    0,    0, 0xff,    0,    0,    0,    0, 0xfe,    0,    0,    0,    0,    0, // 0b0
-	  32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32,   32, // 0c0
-	  32,   32,   32,   32,   32,   32,   32,    0,   32,   32,   32,   32,   32,   32,   32, 0xff, // 0d0
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, // 0e0
-	0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,    0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, // 0f0
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 100
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 110
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 120
-	0xfd, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, 0xfe,    1, 0xff,    1, 0xff,    1, 0xff,    1, // 130
-	0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, 0xfe,    1, 0xff,    1, 0xff,    1, 0xff, // 140
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 150
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 160
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, 0xfd,    1, 0xff,    1, 0xff,    1, 0xff, 0xff, // 170
-	0xff,  210,    1, 0xff,    1, 0xff,  206,    1, 0xff,  205,  205,    1, 0xff, 0xfe,   79,  202, // 180
-	 203,    1, 0xff,  205,  207, 0xff,  211,  209,    1, 0xff, 0xff, 0xfe,  211,  213, 0xff,  214, // 190
-	   1, 0xff,    1, 0xff,    1, 0xff,  218,    1, 0xff,  218, 0xfe, 0xfe,    1, 0xff,  218,    1, // 1a0
-	0xff,  217,  217,    1, 0xff,    1, 0xff,  219,    1, 0xff, 0xfe, 0xfe,    1, 0xff, 0xfe, 0xff, // 1b0
-	0xfe, 0xfe, 0xfe, 0xfe,    2, 0xff, 0xff,    2, 0xff, 0xff,    2, 0xff, 0xff,    1, 0xff,    1, // 1c0
-	0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, 0xff,    1, 0xff, // 1d0
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 1e0
-	0xfe,    2, 0xff, 0xff,    1, 0xff, 0xfd, 0xfd,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 1f0
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 200
-	   1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 210
-	0xfd, 0xfe,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff, // 220
-	   1, 0xff,    1, 0xff, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfd,    1, 0xff, 0xfd, 0xfd, 0xfe, // 230
-	0xfe,    1, 0xff, 0xfd,   69,   71,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff,    1, 0xff  // 240
-};
-
-static const short wchar_tolower[] = {
-	0x130, 0x069,
-	0x178, 0x0ff,
-	0x1f6, 0x195,
-	0x1f7, 0x1bf,
-	0x220, 0x19e,
-	0x23a, 0x2c65,
-	0x23d, 0x19a,
-	0x23e, 0x2c66,
-	0x243, 0x180,
-	0, 0
-};
-
-static const short wchar_toupper[] = {
-	0x0b5, 0x39c,
-	0x0df, 0x0df,
-	0x0ff, 0x178,
-	0x131, 0x049,
-	0x17f, 0x053,
-	0x180, 0x243,
-	0x195, 0x1f6,
-	0x19a, 0x23d,
-	0x19e, 0x220,
-	0x1bf, 0x1f7,
-	0x1c6, 0x1c4,
-	0x1c9, 0x1c7,
-	0x1cc, 0x1ca,
-	0x1dd, 0x18e,
-	0x1f3, 0x1f1,
-	0, 0
-};
-
-#ifdef NEED_WCHAR_FUNCTIONS
-
-// use ctype.h functions for Latin1 (character < 0x100)
-int iswalpha(int c)
-{
-	if (c < 0x80)
-		return isalpha(c);
-	if ((c > 0x3040) && (c <= 0xa700))
-		return 1;  // japanese, chinese characters
-	if (c > MAX_WALPHA)
-		return 0;
-	return walpha_tab[c-0x80];
-}
-
-int iswdigit(int c)
-{
-	if (c < 0x80)
-		return isdigit(c);
-	return 0;
-}
-
-int iswalnum(int c)
-{
-	if (iswdigit(c))
-		return 1;
-	return iswalpha(c);
-}
-
-int towlower(int c)
-{
-	int x;
-	int ix;
-
-	if (c < 0x80)
-		return tolower(c);
-
-	if ((c > MAX_WALPHA) || ((x = walpha_tab[c-0x80]) >= 0xfe))
-		return c;
-
-	if (x == 0xfd) {
-		// special cases, lookup translation table
-		for (ix = 0; wchar_tolower[ix] != 0; ix += 2) {
-			if (wchar_tolower[ix] == c)
-				return wchar_tolower[ix+1];
-		}
-	}
-	return c + x; // convert to lower case
-}
-
-int towupper(int c)
-{
-	int ix;
-	// check whether a previous character code is the upper-case equivalent of this character
-	if (towlower(c-32) == c)
-		return c-32; // yes, use it
-	if (towlower(c-1) == c)
-		return c-1;
-	for (ix = 0; wchar_toupper[ix] != 0; ix += 2) {
-		if (wchar_toupper[ix] == c)
-			return wchar_toupper[ix+1];
-	}
-	return c;  // no
-}
-
-int iswupper(int c)
-{
-	int x;
-	if (c < 0x80)
-		return isupper(c);
-	if (((c > MAX_WALPHA) || (x = walpha_tab[c-0x80]) == 0) || (x == 0xff))
-		return 0;
-	return 1;
-}
-
-int iswlower(int c)
-{
-	if (c < 0x80)
-		return islower(c);
-	if ((c > MAX_WALPHA) || (walpha_tab[c-0x80] != 0xff))
-		return 0;
-	return 1;
-}
-
-int iswspace(int c)
-{
-	if (c < 0x80)
-		return isspace(c);
-	if (c == 0xa0)
-		return 1;
-	return 0;
-}
-
-int iswpunct(int c)
-{
-	if (c < 0x100)
-		return ispunct(c);
-	return 0;
-}
-
-const wchar_t *wcschr(const wchar_t *str, int c)
-{
-	while (*str != 0) {
-		if (*str == c)
-			return str;
-		str++;
-	}
-	return NULL;
-}
-
-#ifndef WINCE
-// wcslen() is provided by WINCE, but not the other wchar functions
-const int wcslen(const wchar_t *str)
-{
-	int ix = 0;
-
-	while (*str != 0)
-		ix++;
-	return ix;
-}
-#endif
-
-float wcstod(const wchar_t *str, wchar_t **tailptr)
-{
-	int ix;
-	char buf[80];
-	while (isspace(*str)) str++;
-	for (ix = 0; ix < 80; ix++) {
-		buf[ix] = str[ix];
-		if (isspace(buf[ix]))
-			break;
-	}
-	*tailptr = (wchar_t *)&str[ix];
-	return atof(buf);
-}
-#endif
-
-// use internal data for iswalpha up to U+024F
-// iswalpha() on Windows is unreliable  (U+AA, U+BA).
-int iswalpha2(int c)
-{
-	if (c < 0x80)
-		return isalpha(c);
-	if ((c > 0x3040) && (c <= 0xa700))
-		return 1; // japanese, chinese characters
-	if (c > MAX_WALPHA)
-		return iswalpha(c);
-	return walpha_tab[c-0x80];
-}
-
-int iswlower2(int c)
-{
-	if (c < 0x80)
-		return islower(c);
-	if (c > MAX_WALPHA)
-		return iswlower(c);
-	if (walpha_tab[c-0x80] == 0xff)
-		return 1;
-	return 0;
-}
-
-int iswupper2(int c)
-{
-	int x;
-	if (c < 0x80)
-		return isupper(c);
-	if (c > MAX_WALPHA)
-		return iswupper(c);
-	if (((x = walpha_tab[c-0x80]) > 0) && (x < 0xfe))
-		return 1;
-	return 0;
-}
-
 int towlower2(unsigned int c)
 {
-	int x;
-	int ix;
-
 	// check for non-standard upper to lower case conversions
-	if (c == 'I') {
-		if (translator->langopts.dotless_i)
-			c = 0x131; // I -> ı
-	}
+	if (c == 'I' && translator->langopts.dotless_i)
+		return 0x131; // I -> ı
 
-	if (c < 0x80)
-		return tolower(c);
-
-	if (c > MAX_WALPHA)
-		return towlower(c);
-
-	if ((x = walpha_tab[c-0x80]) >= 0xfe)
-		return c; // this is not an upper case letter
-
-	if (x == 0xfd) {
-		// special cases, lookup translation table
-		for (ix = 0; wchar_tolower[ix] != 0; ix += 2) {
-			if (wchar_tolower[ix] == (int)c)
-				return wchar_tolower[ix+1];
-		}
-	}
-	return c + x; // convert to lower case
-}
-
-int towupper2(unsigned int c)
-{
-	int ix;
-	if (c > MAX_WALPHA)
-		return towupper(c);
-
-	// check whether a previous character code is the upper-case equivalent of this character
-	if (towlower2(c-32) == (int)c)
-		return c-32; // yes, use it
-	if (towlower2(c-1) == (int)c)
-		return c-1;
-	for (ix = 0; wchar_toupper[ix] != 0; ix += 2) {
-		if (wchar_toupper[ix] == (int)c)
-			return wchar_toupper[ix+1];
-	}
-	return c; // no
+	return ucd_tolower(c);
 }
 
 static int IsRomanU(unsigned int c)
@@ -516,148 +173,26 @@ static int IsRomanU(unsigned int c)
 	return 0;
 }
 
-static void GetC_unget(int c)
-{
-	// This is only called with UTF8 input, not wchar input
-	if (f_input != NULL)
-		ungetc(c, f_input);
-	else {
-		p_textinput--;
-		*p_textinput = c;
-		end_of_input = 0;
-	}
-}
-
 int Eof(void)
 {
 	if (ungot_char != 0)
 		return 0;
 
-	if (f_input != 0)
-		return feof(f_input);
-
-	return end_of_input;
-}
-
-static int GetC_get(void)
-{
-	unsigned int c;
-	unsigned int c2;
-
-	if (f_input != NULL) {
-		c = fgetc(f_input);
-		if (feof(f_input)) c = ' ';
-
-		if (option_multibyte == espeakCHARS_16BIT) {
-			c2 = fgetc(f_input);
-			if (feof(f_input)) c2 = 0;
-			c = c + (c2 << 8);
-		}
-		return c;
-	}
-
-	if (option_multibyte == espeakCHARS_WCHAR) {
-		if (*p_wchar_input == 0) {
-			end_of_input = 1;
-			return 0;
-		}
-
-		if (!end_of_input)
-			return *p_wchar_input++;
-	} else {
-		if (*p_textinput == 0) {
-			end_of_input = 1;
-			return 0;
-		}
-
-		if (!end_of_input) {
-			if (option_multibyte == espeakCHARS_16BIT) {
-				c = p_textinput[0] + (p_textinput[1] << 8);
-				p_textinput += 2;
-				return c;
-			}
-			return *p_textinput++ & 0xff;
-		}
-	}
-	return 0;
+	return text_decoder_eof(p_decoder);
 }
 
 static int GetC(void)
 {
-	// Returns a unicode wide character
-	// Performs UTF8 checking and conversion
-
-	int c;
 	int c1;
-	int c2;
-	int cbuf[4];
-	int ix;
-	int n_bytes;
 	static int ungot2 = 0;
-	static const unsigned char mask[4] = { 0xff, 0x1f, 0x0f, 0x07 };
 
 	if ((c1 = ungot_char) != 0) {
 		ungot_char = 0;
 		return c1;
 	}
 
-	if (ungot2 != 0) {
-		c1 = ungot2;
-		ungot2 = 0;
-	} else
-		c1 = GetC_get();
-
-	if ((option_multibyte == espeakCHARS_WCHAR) || (option_multibyte == espeakCHARS_16BIT)) {
-		count_characters++;
-		return c1; // wchar_t  text
-	}
-
-	if ((option_multibyte < 2) && (c1 & 0x80)) {
-		// multi-byte utf8 encoding, convert to unicode
-		n_bytes = 0;
-
-		if (((c1 & 0xe0) == 0xc0) && ((c1 & 0x1e) != 0))
-			n_bytes = 1;
-		else if ((c1 & 0xf0) == 0xe0)
-			n_bytes = 2;
-		else if (((c1 & 0xf8) == 0xf0) && ((c1 & 0x0f) <= 4))
-			n_bytes = 3;
-
-		if ((ix = n_bytes) > 0) {
-			c = c1 & mask[ix];
-			while (ix > 0) {
-				if ((c2 = cbuf[ix] = GetC_get()) == 0) {
-					if (option_multibyte == espeakCHARS_AUTO)
-						option_multibyte = espeakCHARS_8BIT; // change "auto" option to "no"
-					GetC_unget(' ');
-					break;
-				}
-
-				if ((c2 & 0xc0) != 0x80) {
-					// This is not UTF8.  Change to 8-bit characterset.
-					if ((n_bytes == 2) && (ix == 1))
-						ungot2 = cbuf[2];
-					GetC_unget(c2);
-					break;
-				}
-				c = (c << 6) + (c2 & 0x3f);
-				ix--;
-			}
-			if (ix == 0) {
-				count_characters++;
-				return c;
-			}
-		}
-		// top-bit-set character is not utf8, drop through to 8bit charset case
-		if ((option_multibyte == espeakCHARS_AUTO) && !Eof())
-			option_multibyte = espeakCHARS_8BIT; // change "auto" option to "no"
-	}
-
-	// 8 bit character set, convert to unicode if
 	count_characters++;
-	if (c1 >= 0xa0)
-		return translator->charset_a0[c1-0xa0];
-	return c1;
+	return text_decoder_getc(p_decoder);
 }
 
 static void UngetC(int c)
@@ -847,6 +382,10 @@ static espeak_ng_STATUS LoadSoundFile(const char *fname, int index, espeak_ng_ER
 	}
 
 	length = GetFileLength(fname);
+	if (length < 0) { // length == -errno
+		fclose(f);
+		return create_file_error_context(context, -length, fname);
+	}
 	if (fseek(f, 0, SEEK_SET) == -1) {
 		int error = errno;
 		fclose(f);
@@ -1004,19 +543,19 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 	if (c1 == '-')
 		return CLAUSE_NONE; // no pause
 
-	attributes = punct_attributes[lookupwchar(punct_chars, c1)];
+	attributes = clause_type_from_codepoint(c1);
 
 	short_pause = CLAUSE_SHORTFALL;
-	if ((attributes & CLAUSE_BITS_INTONATION) == 0x1000)
+	if ((attributes & CLAUSE_INTONATION_TYPE) == 0x1000)
 		short_pause = CLAUSE_SHORTCOMMA;
 
 	if ((bufix1 > 0) && !(tr->langopts.param[LOPT_ANNOUNCE_PUNCT] & 2)) {
-		if ((attributes & ~0x8000) == CLAUSE_SEMICOLON)
+		if ((attributes & ~CLAUSE_OPTIONAL_SPACE_AFTER) == CLAUSE_SEMICOLON)
 			return CLAUSE_SHORTFALL;
 		return short_pause;
 	}
 
-	if (attributes & CLAUSE_BIT_SENTENCE)
+	if (attributes & CLAUSE_TYPE_SENTENCE)
 		return attributes;
 
 	return short_pause;
@@ -1112,7 +651,7 @@ static const char *VoiceFromStack()
 			voice_name_specified = 1;
 			strcpy(voice_name, sp->voice_name);
 			language[0] = 0;
-			voice_select.gender = 0;
+			voice_select.gender = ENGENDER_UNKNOWN;
 			voice_select.age = 0;
 			voice_select.variant = 0;
 		}
@@ -1133,7 +672,7 @@ static const char *VoiceFromStack()
 			if (voice_name_specified == 0)
 				voice_name[0] = 0; // forget a previous voice name if a language is specified
 		}
-		if (sp->voice_gender != 0)
+		if (sp->voice_gender != ENGENDER_UNKNOWN)
 			voice_select.gender = sp->voice_gender;
 
 		if (sp->voice_age != 0)
@@ -1148,7 +687,7 @@ static const char *VoiceFromStack()
 	if (v_id == NULL)
 		return "default";
 
-	if ((strchr(v_id, '+') == NULL) && ((voice_select.gender == 0) || (voice_select.gender == base_voice.gender)) && (base_voice_variant_name[0] != 0)) {
+	if ((strchr(v_id, '+') == NULL) && ((voice_select.gender == ENGENDER_UNKNOWN) || (voice_select.gender == base_voice.gender)) && (base_voice_variant_name[0] != 0)) {
 		// a voice variant has not been selected, use the original voice variant
 		sprintf(buf, "%s+%s", v_id, base_voice_variant_name);
 		strncpy0(voice_name, buf, sizeof(voice_name));
@@ -1303,7 +842,7 @@ static int attrnumber(const wchar_t *pw, int default_value, int type)
 
 	while (IsDigit09(*pw))
 		value = value*10 + *pw++ - '0';
-	if ((type == 1) && (towlower(*pw) == 's')) {
+	if ((type == 1) && (ucd_tolower(*pw) == 's')) {
 		// time: seconds rather than ms
 		value *= 1000;
 	}
@@ -1438,7 +977,7 @@ static int GetVoiceAttributes(wchar_t *pw, int tag_type)
 	// a voice change.
 	// If it's a closing tag, delete the top frame of the stack and determine whether this implies
 	// a voice change.
-	// Returns  CLAUSE_BIT_VOICE if there is a voice change
+	// Returns  CLAUSE_TYPE_VOICE_CHANGE if there is a voice change
 
 	wchar_t *lang;
 	wchar_t *gender;
@@ -1450,10 +989,10 @@ static int GetVoiceAttributes(wchar_t *pw, int tag_type)
 	SSML_STACK *ssml_sp;
 
 	static const MNEM_TAB mnem_gender[] = {
-		{ "male", 1 },
-		{ "female", 2 },
-		{ "neutral", 3 },
-		{ NULL, 0 }
+		{ "male", ENGENDER_MALE },
+		{ "female", ENGENDER_FEMALE },
+		{ "neutral", ENGENDER_NEUTRAL },
+		{ NULL, ENGENDER_UNKNOWN }
 	};
 
 	if (tag_type & SSML_CLOSE) {
@@ -1496,7 +1035,7 @@ static int GetVoiceAttributes(wchar_t *pw, int tag_type)
 	if (strcmp(new_voice_id, current_voice_id) != 0) {
 		// add an embedded command to change the voice
 		strcpy(current_voice_id, new_voice_id);
-		return CLAUSE_BIT_VOICE;    // change of voice
+		return CLAUSE_TYPE_VOICE_CHANGE;
 	}
 
 	return 0;
@@ -1997,7 +1536,7 @@ static MNEM_TAB xml_char_mnemonics[] = {
 	{ NULL,   -1 }
 };
 
-int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change)
+int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change)
 {
 	/* Find the end of the current clause.
 	    Write the clause into  buf
@@ -2024,7 +1563,6 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 	int phoneme_mode = 0;
 	int n_xml_buf;
 	int terminator;
-	int punct;
 	int found;
 	int any_alnum = 0;
 	int self_closing;
@@ -2049,12 +1587,9 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 	tr->phonemes_repeat_count = 0;
 	tr->clause_upper_count = 0;
 	tr->clause_lower_count = 0;
-	end_of_input = 0;
 	*tone_type = 0;
 	*voice_change = 0;
 
-	f_input = f_in; // for GetC etc
-
 	if (ungot_word != NULL) {
 		strcpy(buf, ungot_word);
 		ix += strlen(ungot_word);
@@ -2069,7 +1604,6 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 	while (!Eof() || (ungot_char != 0) || (ungot_char2 != 0) || (ungot_string_ix >= 0)) {
 		if (!iswalnum(c1)) {
 			if ((end_character_position > 0) && (count_characters > end_character_position)) {
-				end_of_input = 1;
 				return CLAUSE_EOF;
 			}
 
@@ -2147,7 +1681,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 					while (!Eof() && (c1 != '>'))
 						c1 = GetC();
 					c2 = ' ';
-				} else if ((c2 == '/') || iswalpha2(c2)) {
+				} else if ((c2 == '/') || iswalpha(c2)) {
 					// check for space in the output buffer for embedded commands produced by the SSML tag
 					if (ix > (n_buf - 20)) {
 						// Perhaps not enough room, end the clause before the SSML tag
@@ -2184,7 +1718,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 						buf[ix] = ' ';
 						buf[ix++] = 0;
 
-						if (terminator & CLAUSE_BIT_VOICE)
+						if (terminator & CLAUSE_TYPE_VOICE_CHANGE)
 							strcpy(voice_change, current_voice_id);
 						return terminator;
 					}
@@ -2201,13 +1735,12 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 
 		if ((c2 == '\n') && (option_linelength == -1)) {
 			// single-line mode, return immediately on NL
-			if ((punct = lookupwchar(punct_chars, c1)) == 0) {
+			if ((terminator = clause_type_from_codepoint(c1)) == CLAUSE_NONE) {
 				charix[ix] = count_characters - clause_start_char;
 				*charix_top = ix;
 				ix += utf8_out(c1, &buf[ix]);
 				terminator = CLAUSE_PERIOD; // line doesn't end in punctuation, assume period
-			} else
-				terminator = punct_attributes[punct];
+			}
 			buf[ix] = ' ';
 			buf[ix+1] = 0;
 			return terminator;
@@ -2299,9 +1832,9 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 			}
 		}
 
-		if (iswupper2(c1)) {
+		if (iswupper(c1)) {
 			tr->clause_upper_count++;
-			if ((option_capitals == 2) && (sayas_mode == 0) && !iswupper2(cprev)) {
+			if ((option_capitals == 2) && (sayas_mode == 0) && !iswupper(cprev)) {
 				char text_buf[40];
 				char text_buf2[30];
 				if (LookupSpecial(tr, "_cap", text_buf2) != NULL) {
@@ -2313,7 +1846,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 					}
 				}
 			}
-		} else if (iswalpha2(c1))
+		} else if (iswalpha(c1))
 			tr->clause_lower_count++;
 
 		if (option_phoneme_input) {
@@ -2370,7 +1903,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 				// i.e. is dot followed by an upper-case letter?
 
 				if (!iswspace(c1)) {
-					if (!IsAlpha(c1) || !iswlower2(c1)) {
+					if (!IsAlpha(c1) || !iswlower(c1)) {
 						UngetC(c2);
 						ungot_char2 = c1;
 						buf[end_clause_index] = ' '; // delete the end-clause punctuation
@@ -2394,17 +1927,15 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 			}
 
 			punct_data = 0;
-			if ((punct = lookupwchar(punct_chars, c1)) != 0) {
-				punct_data = punct_attributes[punct];
-
-				if (punct_data & PUNCT_IN_WORD) {
+			if ((punct_data = clause_type_from_codepoint(c1)) != CLAUSE_NONE) {
+				if (punct_data & CLAUSE_PUNCTUATION_IN_WORD) {
 					// Armenian punctuation inside a word
 					stressed_word = 1;
 					*tone_type = punct_data >> 12 & 0xf; // override the end-of-sentence type
 					continue;
 				}
 
-				if ((iswspace(c2) || (punct_data & 0x8000) || IsBracket(c2) || (c2 == '?') || Eof() || (c2 == ctrl_embedded))) { // don't check for '-' because it prevents recognizing ':-)'
+				if ((iswspace(c2) || (punct_data & CLAUSE_OPTIONAL_SPACE_AFTER) || IsBracket(c2) || (c2 == '?') || Eof() || (c2 == ctrl_embedded))) { // don't check for '-' because it prevents recognizing ':-)'
 					// note: (c2='?') is for when a smart-quote has been replaced by '?'
 					is_end_clause = 1;
 				}
@@ -2423,7 +1954,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 				}
 			}
 
-			if ((punct_data & PUNCT_SAY_NAME) && (announced_punctuation == 0)) {
+			if ((punct_data & CLAUSE_SPEAK_PUNCTUATION_NAME) && (announced_punctuation == 0)) {
 				// used for elipsis (and 3 dots) if a pronunciation for elipsis is given in *_list
 				char *p2;
 
@@ -2432,7 +1963,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 				if (p2[0] != 0) {
 					ix += strlen(p2);
 					announced_punctuation = c1;
-					punct_data = punct_data & ~CLAUSE_BITS_INTONATION; // change intonation type to 0 (full-stop)
+					punct_data = punct_data & ~CLAUSE_INTONATION_TYPE; // change intonation type to 0 (full-stop)
 				}
 			}
 
@@ -2449,10 +1980,10 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 				}
 
 				if ((c1 == '.') && (nl_count < 2))
-					punct_data |= CLAUSE_DOT;
+					punct_data |= CLAUSE_DOT_AFTER_LAST_WORD;
 
 				if (nl_count == 0) {
-					if ((c1 == ',') && (cprev == '.') && (tr->translator_name == L('h', 'u')) && iswdigit(cprev2) && (iswdigit(c_next) || (iswlower2(c_next)))) {
+					if ((c1 == ',') && (cprev == '.') && (tr->translator_name == L('h', 'u')) && iswdigit(cprev2) && (iswdigit(c_next) || (iswlower(c_next)))) {
 						// lang=hu, fix for ordinal numbers, eg:  "december 2., szerda", ignore ',' after ordinal number
 						c1 = CHAR_COMMA_BREAK;
 						is_end_clause = 0;
@@ -2464,11 +1995,11 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 							// dot after a number indicates an ordinal number
 							if (!iswdigit(cprev))
 								is_end_clause = 0; // Roman number followed by dot
-							else if (iswlower2(c_next) || (c_next == '-')) // hyphen is needed for lang-hu (eg. 2.-kal)
+							else if (iswlower(c_next) || (c_next == '-')) // hyphen is needed for lang-hu (eg. 2.-kal)
 								is_end_clause = 0; // only if followed by lower-case, (or if there is a XML tag)
 						} else if (c_next == '\'')
 							is_end_clause = 0;    // eg. u.s.a.'s
-						if (iswlower2(c_next)) {
+						if (iswlower(c_next)) {
 							// next word has no capital letter, this dot is probably from an abbreviation
 							is_end_clause = 0;
 						}
@@ -2499,7 +2030,7 @@ int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix
 					buf[ix+1] = 0;
 
 					if (iswdigit(cprev) && !IsAlpha(c_next)) // ????
-						punct_data &= ~CLAUSE_DOT;
+						punct_data &= ~CLAUSE_DOT_AFTER_LAST_WORD;
 					if (nl_count > 1) {
 						if ((punct_data == CLAUSE_QUESTION) || (punct_data == CLAUSE_EXCLAMATION))
 							return punct_data + 35; // with a longer pause
