@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 to 2013 by Jonathan Duddington
  * email: jonsd@users.sourceforge.net
- * Copyright (C) 2013-2016 Reece H. Dunn
+ * Copyright (C) 2013-2017 Reece H. Dunn
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
 
 #include <espeak-ng/espeak_ng.h>
 #include <espeak-ng/speak_lib.h>
+#include <espeak-ng/encoding.h>
 
 #include "speech.h"
 #include "phoneme.h"
@@ -57,6 +58,7 @@
 #include "event.h"
 
 unsigned char *outbuf = NULL;
+int outbuf_size = 0;
 
 espeak_EVENT *event_list = NULL;
 int event_list_ix = 0;
@@ -81,6 +83,15 @@ int (*phoneme_callback)(const char *) = NULL;
 char path_home[N_PATH_HOME]; // this is the espeak-ng-data directory
 extern int saved_parameters[N_SPEECH_PARAM]; // Parameters saved on synthesis start
 
+void cancel_audio(void)
+{
+#ifdef HAVE_PCAUDIOLIB_AUDIO_H
+	if ((my_mode & ENOUTPUT_MODE_SPEAK_AUDIO) == ENOUTPUT_MODE_SPEAK_AUDIO) {
+		audio_object_flush(my_audio);
+	}
+#endif
+}
+
 static int dispatch_audio(short *outbuf, int length, espeak_EVENT *event)
 {
 	int a_wave_can_be_played = 1;
@@ -223,14 +234,14 @@ static int check_data_path(const char *path, int allow_directory)
 	if (!path) return 0;
 
 	snprintf(path_home, sizeof(path_home), "%s/espeak-ng-data", path);
-	if (GetFileLength(path_home) == -2)
+	if (GetFileLength(path_home) == -EISDIR)
 		return 1;
 
 	if (!allow_directory)
 		return 0;
 
 	snprintf(path_home, sizeof(path_home), "%s", path);
-	return GetFileLength(path_home) == -2;
+	return GetFileLength(path_home) == -EISDIR;
 }
 
 #pragma GCC visibility push(default)
@@ -242,7 +253,8 @@ ESPEAK_NG_API espeak_ng_STATUS espeak_ng_InitializeOutput(espeak_ng_OUTPUT_MODE
 	out_samplerate = 0;
 
 #ifdef HAVE_PCAUDIOLIB_AUDIO_H
-	my_audio = create_audio_device_object(device, "eSpeak", "Text-to-Speech");
+	if (my_audio == NULL)
+		my_audio = create_audio_device_object(device, "eSpeak", "Text-to-Speech");
 #endif
 
 	// buffer_length is in mS, allocate 2 bytes per sample
@@ -272,10 +284,10 @@ int GetFileLength(const char *filename)
 	struct stat statbuf;
 
 	if (stat(filename, &statbuf) != 0)
-		return 0;
+		return -errno;
 
 	if (S_ISDIR(statbuf.st_mode))
-		return -2; // a directory
+		return -EISDIR;
 
 	return statbuf.st_size;
 }
@@ -380,17 +392,27 @@ static espeak_ng_STATUS Synthesize(unsigned int unique_identifier, const void *t
 	if ((outbuf == NULL) || (event_list == NULL))
 		return ENS_NOT_INITIALIZED;
 
-	option_multibyte = flags & 7;
 	option_ssml = flags & espeakSSML;
 	option_phoneme_input = flags & espeakPHONEMES;
 	option_endpause = flags & espeakENDPAUSE;
 
 	count_samples = 0;
 
-	if (translator == NULL)
-		espeak_SetVoiceByName("default");
+	espeak_ng_STATUS status;
+	if (translator == NULL) {
+		status = espeak_ng_SetVoiceByName("en");
+		if (status != ENS_OK)
+			return status;
+	}
 
-	SpeakNextClause(NULL, text, 0);
+	if (p_decoder == NULL)
+		p_decoder = create_text_decoder();
+
+	status = text_decoder_decode_string_multibyte(p_decoder, text, translator->encoding, flags);
+	if (status != ENS_OK)
+		return status;
+
+	SpeakNextClause(0);
 
 	for (;;) {
 		out_ptr = outbuf;
@@ -412,7 +434,7 @@ static espeak_ng_STATUS Synthesize(unsigned int unique_identifier, const void *t
 		} else if (synth_callback)
 			finished = synth_callback((short *)outbuf, length, event_list);
 		if (finished) {
-			SpeakNextClause(NULL, 0, 2); // stop
+			SpeakNextClause(2); // stop
 			return ENS_SPEECH_STOPPED;
 		}
 
@@ -425,7 +447,7 @@ static espeak_ng_STATUS Synthesize(unsigned int unique_identifier, const void *t
 				event_list[0].unique_identifier = my_unique_identifier;
 				event_list[0].user_data = my_user_data;
 
-				if (SpeakNextClause(NULL, NULL, 1) == 0) {
+				if (SpeakNextClause(1) == 0) {
 					finished = 0;
 					if ((my_mode & ENOUTPUT_MODE_SPEAK_AUDIO) == ENOUTPUT_MODE_SPEAK_AUDIO) {
 						if (dispatch_audio(NULL, 0, NULL) < 0)
@@ -433,7 +455,7 @@ static espeak_ng_STATUS Synthesize(unsigned int unique_identifier, const void *t
 					} else if (synth_callback)
 						finished = synth_callback(NULL, 0, event_list); // NULL buffer ptr indicates end of data
 					if (finished) {
-						SpeakNextClause(NULL, 0, 2); // stop
+						SpeakNextClause(2); // stop
 						return ENS_SPEECH_STOPPED;
 					}
 					return ENS_OK;
@@ -803,8 +825,15 @@ ESPEAK_API const char *espeak_TextToPhonemes(const void **textptr, int textmode,
 	    bits 8-23:  separator character, between phoneme names
 	 */
 
-	option_multibyte = textmode & 7;
-	*textptr = TranslateClause(translator, NULL, *textptr, NULL, NULL);
+	if (p_decoder == NULL)
+		p_decoder = create_text_decoder();
+
+	if (text_decoder_decode_string_multibyte(p_decoder, *textptr, translator->encoding, textmode) != ENS_OK)
+		return NULL;
+
+	TranslateClause(translator, NULL, NULL);
+	*textptr = text_decoder_get_buffer(p_decoder);
+
 	return GetTranslatedPhonemeString(phonememode);
 }
 
@@ -862,6 +891,7 @@ ESPEAK_NG_API espeak_ng_STATUS espeak_ng_Terminate(void)
 #ifdef HAVE_PCAUDIOLIB_AUDIO_H
 		audio_object_close(my_audio);
 		audio_object_destroy(my_audio);
+		my_audio = NULL;
 #endif
 		out_samplerate = 0;
 	}
@@ -872,6 +902,12 @@ ESPEAK_NG_API espeak_ng_STATUS espeak_ng_Terminate(void)
 	outbuf = NULL;
 	FreePhData();
 	FreeVoiceList();
+	translator = NULL;
+
+	if (p_decoder != NULL) {
+		destroy_text_decoder(p_decoder);
+		p_decoder = NULL;
+	}
 
 	return ENS_OK;
 }
