@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 to 2014 by Jonathan Duddington
  * email: jonsd@users.sourceforge.net
- * Copyright (C) 2015-2016 Reece H. Dunn
+ * Copyright (C) 2015-2017 Reece H. Dunn
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -268,9 +268,8 @@ typedef struct {
 
 // instructions
 
-#define i_RETURN        0x0001
-#define i_CONTINUE      0x0002
-#define i_NOT           0x0003
+#define OPCODE_RETURN        0x0001
+#define OPCODE_CONTINUE      0x0002
 
 // Group 0 instrcutions with 8 bit operand.  These values go into bits 8-15 of the instruction
 #define i_CHANGE_PHONEME 0x01
@@ -292,6 +291,7 @@ typedef struct {
 // conditions and jumps
 #define i_CONDITION  0x2000
 #define i_OR         0x1000  // added to i_CONDITION
+#define i_NOT        0x0003
 
 #define i_JUMP       0x6000
 #define i_JUMP_FALSE 0x6800
@@ -312,36 +312,29 @@ typedef struct {
 #define i_WAVADD     0xf000
 
 // conditions
-#define i_isDiminished 0x80
-#define i_isUnstressed 0x81
-#define i_isNotStressed 0x82
-#define i_isStressed   0x83
-#define i_isMaxStress  0x84
+#define CONDITION_IS_PHONEME_TYPE 0x00
+#define CONDITION_IS_PLACE_OF_ARTICULATION 0x20
+#define CONDITION_IS_PHFLAG_SET 0x40
+#define CONDITION_IS_OTHER 0x80
 
-#define i_isBreak      0x85
-#define i_isWordStart  0x86
-#define i_notWordStart 0x87
-#define i_isWordEnd    0x88
-#define i_isAfterStress 0x89
-#define i_isNotVowel   0x8a
-#define i_isFinalVowel 0x8b
-#define i_isVoiced     0x8c
-#define i_isFirstVowel 0x8d
-#define i_isSecondVowel 0x8e
-#define i_isSeqFlag1   0x8f
-#define i_IsTranslationGiven 0x90
+// other conditions (stress)
+#define isDiminished   0
+#define isUnstressed   1
+#define isNotStressed  2
+#define isStressed     3
+#define isMaxStress    4
 
-// place of articulation
-#define i_isVel      0x28
-
-// phflags
-#define i_isSibilant   0x45 // bit  5 in phflags
-#define i_isPalatal    0x49 // bit  9 in phflags
-#define i_isLong       0x55 // bit 21 in phflags
-#define i_isRhotic     0x57 // bit 23 in phflags
-#define i_isFlag1      0x5c
-#define i_isFlag2      0x5d
-#define i_isFlag3      0x5e
+// other conditions
+#define isBreak        5 // pause phoneme or (stop/vstop/vfric not followed by vowel or (liquid in same word))
+#define isWordStart    6
+#define isWordEnd      8
+#define isAfterStress  9
+#define isNotVowel    10
+#define isFinalVowel  11
+#define isVoiced      12 // voiced consonant, or vowel
+#define isFirstVowel  13
+#define isSecondVowel 14
+#define isTranslationGiven 16 // phoneme translation given in **_list or as [[...]]
 
 #define i_StressLevel  0x800
 
@@ -499,7 +492,7 @@ espeak_ng_STATUS LoadPhData(int *srate, espeak_ng_ERROR_CONTEXT *context);
 void SynthesizeInit(void);
 int  Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume);
 void MakeWave2(PHONEME_LIST *p, int n_ph);
-int  SpeakNextClause(FILE *f_text, const void *text_in, int control);
+int  SpeakNextClause(int control);
 void SetSpeed(int control);
 void SetEmbedded(int control, int value);
 void SelectPhonemeTable(int number);
@@ -519,7 +512,6 @@ extern int formant_rate[];         // max rate of change of each formant
 extern SPEED_FACTORS speed;
 
 extern long count_samples;
-extern int outbuf_size;
 extern unsigned char *out_ptr;
 extern unsigned char *out_start;
 extern unsigned char *out_end;
