@@ -34,7 +34,6 @@ extern "C"
 
 #define N_RULE_GROUP2    120 // max num of two-letter rule chains
 #define N_HASH_DICT     1024
-#define N_CHARSETS        20
 #define N_LETTER_GROUPS   95 // maximum is 127-32
 
 // dictionary flags, word 1
@@ -199,38 +198,46 @@ extern "C"
 #define LETTERGP_Y      6
 #define LETTERGP_VOWEL2 7
 
-// Punctuation types  returned by ReadClause()
-// bits 0-11 pause x 10mS
-// bits12-14 intonation type
-// bit 15- don't need space after the punctuation
-// bit 19=sentence, bit 18=clause,  bits 17=voice change
-// bit 16 used to distinguish otherwise identical types
-// bit 20= punctuation character can be inside a word (Armenian)
-// bit 21= speak the name of the punctuation character
-// bit 22= dot after the last word
-// bit 23= pause is x 320mS (not x 10mS)
+// Punctuation types returned by ReadClause()
+//@{
 
-#define CLAUSE_BIT_SENTENCE     0x80000
-#define CLAUSE_BIT_CLAUSE       0x40000
-#define CLAUSE_BIT_VOICE        0x20000
-#define CLAUSE_BITS_INTONATION   0x7000
-#define PUNCT_IN_WORD          0x100000
-#define PUNCT_SAY_NAME         0x200000
-#define CLAUSE_DOT             0x400000
-#define CLAUSE_PAUSE_LONG      0x800000
+#define CLAUSE_PAUSE                  0x00000FFF // pause (x 10mS)
+#define CLAUSE_INTONATION_TYPE        0x00007000 // intonation type
+#define CLAUSE_OPTIONAL_SPACE_AFTER   0x00008000 // don't need space after the punctuation
+#define CLAUSE_TYPE                   0x000F0000 // phrase type
+#define CLAUSE_PUNCTUATION_IN_WORD    0x00100000 // punctuation character can be inside a word (Armenian)
+#define CLAUSE_SPEAK_PUNCTUATION_NAME 0x00200000 // speak the name of the punctuation character
+#define CLAUSE_DOT_AFTER_LAST_WORD    0x00400000 // dot after the last word
+#define CLAUSE_PAUSE_LONG             0x00800000 // x 320mS to the CLAUSE_PAUSE value
 
-#define CLAUSE_NONE        ( 0 + 0x04000)
-#define CLAUSE_PARAGRAPH   (70 + 0x80000)
-#define CLAUSE_EOF         (40 + 0x90000)
-#define CLAUSE_VOICE       ( 0 + 0x24000)
-#define CLAUSE_PERIOD      (40 + 0x80000)
-#define CLAUSE_COMMA       (20 + 0x41000)
-#define CLAUSE_SHORTCOMMA  ( 4 + 0x41000)
-#define CLAUSE_SHORTFALL   ( 4 + 0x40000)
-#define CLAUSE_QUESTION    (40 + 0x82000)
-#define CLAUSE_EXCLAMATION (45 + 0x83000)
-#define CLAUSE_COLON       (30 + 0x40000)
-#define CLAUSE_SEMICOLON   (30 + 0x41000)
+#define CLAUSE_INTONATION_FULL_STOP   0x00000000
+#define CLAUSE_INTONATION_COMMA       0x00001000
+#define CLAUSE_INTONATION_QUESTION    0x00002000
+#define CLAUSE_INTONATION_EXCLAMATION 0x00003000
+#define CLAUSE_INTONATION_NONE        0x00004000
+
+#define CLAUSE_TYPE_NONE              0x00000000
+#define CLAUSE_TYPE_EOF               0x00010000
+#define CLAUSE_TYPE_VOICE_CHANGE      0x00020000
+#define CLAUSE_TYPE_CLAUSE            0x00040000
+#define CLAUSE_TYPE_SENTENCE          0x00080000
+
+#define CLAUSE_NONE        ( 0 | CLAUSE_INTONATION_NONE        | CLAUSE_TYPE_NONE)
+#define CLAUSE_PARAGRAPH   (70 | CLAUSE_INTONATION_FULL_STOP   | CLAUSE_TYPE_SENTENCE)
+#define CLAUSE_EOF         (40 | CLAUSE_INTONATION_FULL_STOP   | CLAUSE_TYPE_SENTENCE | CLAUSE_TYPE_EOF)
+#define CLAUSE_VOICE       ( 0 | CLAUSE_INTONATION_NONE        | CLAUSE_TYPE_VOICE_CHANGE)
+#define CLAUSE_PERIOD      (40 | CLAUSE_INTONATION_FULL_STOP   | CLAUSE_TYPE_SENTENCE)
+#define CLAUSE_COMMA       (20 | CLAUSE_INTONATION_COMMA       | CLAUSE_TYPE_CLAUSE)
+#define CLAUSE_SHORTCOMMA  ( 4 | CLAUSE_INTONATION_COMMA       | CLAUSE_TYPE_CLAUSE)
+#define CLAUSE_SHORTFALL   ( 4 | CLAUSE_INTONATION_FULL_STOP   | CLAUSE_TYPE_CLAUSE)
+#define CLAUSE_QUESTION    (40 | CLAUSE_INTONATION_QUESTION    | CLAUSE_TYPE_SENTENCE)
+#define CLAUSE_EXCLAMATION (45 | CLAUSE_INTONATION_EXCLAMATION | CLAUSE_TYPE_SENTENCE)
+#define CLAUSE_COLON       (30 | CLAUSE_INTONATION_FULL_STOP   | CLAUSE_TYPE_CLAUSE)
+#define CLAUSE_SEMICOLON   (30 | CLAUSE_INTONATION_COMMA       | CLAUSE_TYPE_CLAUSE)
+
+int clause_type_from_codepoint(uint32_t c);
+
+//@}
 
 #define SAYAS_CHARS        0x12
 #define SAYAS_GLYPHS       0x13
@@ -563,17 +570,12 @@ typedef struct {
 	int testing;            // testing options: bit 1= specify stressed syllable in the form:  "outdoor/2"
 	int listx;    // compile *_listx after *list
 	const unsigned int *replace_chars;      // characters to be substitutes
-	char ascii_language[8];  // switch to this language for Latin characters
 	int our_alphabet;           // offset for main alphabet (if not set in letter_bits_offset)
 	int alt_alphabet;       // offset for another language to recognize
 	int alt_alphabet_lang;  // language for the alt_alphabet
 	int max_lengthmod;
 	int lengthen_tonic;   // lengthen the tonic syllable
 	int suffix_add_e;      // replace a suffix (which has the SUFX_E flag) with this character
-
-#define DICTDIALECT_EN_US  1  // bit number
-#define DICTDIALECT_ES_LA  2
-	int dict_dialect;         // bitmap, use a dialect for foreign words
 } LANGUAGE_OPTIONS;
 
 // a parameter of ChangePhonemes()
@@ -603,7 +605,7 @@ typedef struct {
 	short stress_lengths[8];
 	int dict_condition;    // conditional apply some pronunciation rules and dict.lookups
 	int dict_min_size;
-	const unsigned short *charset_a0;   // unicodes for characters 0xa0 to oxff
+	espeak_ng_ENCODING encoding;
 	const wchar_t *char_plus_apostrophe;  // single chars + apostrophe treated as words
 	const wchar_t *punct_within_word;   // allow these punctuation characters within words
 	const unsigned short *chars_ignore;
@@ -662,7 +664,6 @@ extern int option_tone_flags;
 extern int option_phonemes;
 extern int option_phoneme_events;
 extern int option_linelength;     // treat lines shorter than this as end-of-clause
-extern int option_multibyte;
 extern int option_capitals;
 extern int option_punctuation;
 extern int option_endpause;
@@ -694,11 +695,9 @@ extern unsigned char punctuation_to_tone[INTONATION_TYPES][PUNCT_INTONATIONS];
 
 extern Translator *translator;
 extern Translator *translator2;
-extern const unsigned short *charsets[N_CHARSETS];
 extern char dictionary_name[40];
 extern char ctrl_embedded;    // to allow an alternative CTRL for embedded commands
-extern unsigned char *p_textinput;
-extern wchar_t *p_wchar_input;
+extern espeak_ng_TEXT_DECODER *p_decoder;
 extern int dictionary_skipwords;
 
 extern int (*uri_callback)(int, const char *, const char *);
@@ -724,16 +723,11 @@ int IsDigit09(unsigned int c);
 int IsAlpha(unsigned int c);
 int IsVowel(Translator *tr, int c);
 int IsSuperscript(int letter);
-int iswalpha2(int c);
 int isspace2(unsigned int c);
-int iswlower2(int c);
-int iswupper2(int c);
-int towlower2(unsigned int c);
-int towupper2(unsigned int c);
+int towlower2(unsigned int c); // Supports Turkish I
 const char *GetTranslatedPhonemeString(int phoneme_mode);
 const char *WordToString2(unsigned int word);
 ALPHABET *AlphabetFromChar(int c);
-ALPHABET *AlphabetFromName(const char *name);
 
 Translator *SelectTranslator(const char *name);
 int SetTranslator2(const char *name);
@@ -767,8 +761,8 @@ int Unpronouncable(Translator *tr, char *word, int posn);
 void SetWordStress(Translator *tr, char *output, unsigned int *dictionary_flags, int tonic, int prev_stress);
 int TranslateRules(Translator *tr, char *p, char *phonemes, int size, char *end_phonemes, int end_flags, unsigned int *dict_flags);
 int TranslateWord(Translator *tr, char *word1, WORD_TAB *wtab, char *word_out);
-void *TranslateClause(Translator *tr, FILE *f_text, const void *vp_input, int *tone, char **voice_change);
-int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change);
+void TranslateClause(Translator *tr, int *tone, char **voice_change);
+int ReadClause(Translator *tr, char *buf, short *charix, int *charix_top, int n_buf, int *tone_type, char *voice_change);
 
 void SetVoiceStack(espeak_VOICE *v, const char *variant_name);
 void InterpretPhoneme(Translator *tr, int control, PHONEME_LIST *plist, PHONEME_DATA *phdata, WORD_PH_DATA *worddata);
