@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2005 to 2015 by Jonathan Duddington
  * email: jonsd@users.sourceforge.net
- * Copyright (C) 2015-2016 Reece H. Dunn
+ * Copyright (C) 2015-2017 Reece H. Dunn
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,12 +20,13 @@
 #include "config.h"
 
 #include <ctype.h>
+#include <wctype.h>
+#include <errno.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
-#include <wctype.h>
 
 #if defined(_WIN32) || defined(_WIN64)
 #include <windows.h>
@@ -35,6 +36,7 @@
 
 #include <espeak-ng/espeak_ng.h>
 #include <espeak-ng/speak_lib.h>
+#include <espeak-ng/encoding.h>
 
 #include "speech.h"
 #include "phoneme.h"
@@ -43,10 +45,9 @@
 #include "translate.h"
 
 MNEM_TAB genders[] = {
-	{ "unknown", 0 },
-	{ "male", 1 },
-	{ "female", 2 },
-	{ NULL, 0 }
+	{ "male", ENGENDER_MALE },
+	{ "female", ENGENDER_FEMALE },
+	{ NULL, ENGENDER_MALE }
 };
 
 int tone_points[12] = { 600, 170, 1200, 135, 2000, 110, 3000, 110, -1, 0 };
@@ -59,7 +60,6 @@ int formant_rate[9]; // values adjusted for actual sample rate
 #define N_VOICES_LIST  250
 static int n_voices_list = 0;
 static espeak_VOICE *voices_list[N_VOICES_LIST];
-static int len_path_voices;
 
 espeak_VOICE current_voice_selected;
 
@@ -70,6 +70,7 @@ enum {
 	V_TRANSLATOR,
 	V_PHONEMES,
 	V_DICTIONARY,
+	V_VARIANTS,
 
 	V_MAINTAINER,
 	V_STATUS,
@@ -96,7 +97,6 @@ enum {
 	V_DICTRULES,
 	V_STRESSRULE,
 	V_STRESSOPT,
-	V_CHARSET,
 	V_NUMBERS,
 	V_OPTION,
 
@@ -106,7 +106,6 @@ enum {
 	V_SPEED,
 	V_DICTMIN,
 	V_ALPHABET2,
-	V_DICTDIALECT,
 
 	// these need a phoneme table to have been specified
 	V_REPLACE,
@@ -127,6 +126,7 @@ static MNEM_TAB keyword_tab[] = {
 	{ "maintainer",   V_MAINTAINER },
 	{ "status",       V_STATUS },
 
+	{ "variants",     V_VARIANTS },
 	{ "formant",      V_FORMANT },
 	{ "pitch",        V_PITCH },
 	{ "phonemes",     V_PHONEMES },
@@ -140,7 +140,6 @@ static MNEM_TAB keyword_tab[] = {
 	{ "dictrules",    V_DICTRULES },
 	{ "stressrule",   V_STRESSRULE },
 	{ "stressopt",    V_STRESSOPT },
-	{ "charset",      V_CHARSET },
 	{ "replace",      V_REPLACE },
 	{ "words",        V_WORDGAP },
 	{ "echo",         V_ECHO },
@@ -159,8 +158,6 @@ static MNEM_TAB keyword_tab[] = {
 	{ "fast_test2",   V_FAST },
 	{ "speed",        V_SPEED },
 	{ "dict_min",     V_DICTMIN },
-	{ "alphabet2",    V_ALPHABET2 },
-	{ "dictdialect",  V_DICTDIALECT },
 
 	// these just set a value in langopts.param[]
 	{ "l_dieresis",       0x100+LOPT_DIERESES },
@@ -174,13 +171,6 @@ static MNEM_TAB keyword_tab[] = {
 	{ NULL, 0 }
 };
 
-static MNEM_TAB dict_dialects[] = {
-	{ "en-us", DICTDIALECT_EN_US },
-	{ "es-la", DICTDIALECT_ES_LA },
-
-	{ NULL, 0 }
-};
-
 #define N_VOICE_VARIANTS   12
 const char variants_either[N_VOICE_VARIANTS] = { 1, 2, 12, 3, 13, 4, 14, 5, 11, 0 };
 const char variants_male[N_VOICE_VARIANTS] = { 1, 2, 3, 4, 5, 6, 0 };
@@ -273,10 +263,8 @@ void ReadTonePoints(char *string, int *tone_pts)
 	       &tone_pts[8], &tone_pts[9]);
 }
 
-static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char *leafname)
+static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, int is_language_file)
 {
-	(void)leafname; // unused (except for PLATFORM_WINDOWS)
-
 	// Read a Voice file, allocate a VOICE_DATA and set data from the
 	// file's  language, gender, name  lines
 
@@ -296,33 +284,28 @@ static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char *le
 	int n_variants = 4; // default, number of variants of this voice before using another voice
 	int gender;
 
-#ifdef PLATFORM_WINDOWS
-	char fname_buf[sizeof(path_home)+15];
-	if (memcmp(leafname, "mb-", 3) == 0) {
-		// check whether the mbrola speech data is present for this voice
-		memcpy(vname, &leafname[3], 3);
-		vname[3] = 0;
-		sprintf(fname_buf, "%s/mbrola/%s", path_home, vname);
-
-		if (GetFileLength(fname_buf) <= 0)
-			return 0;
-	}
-#endif
-
 	vname[0] = 0;
 	vgender[0] = 0;
 	age = 0;
 
 	while (fgets_strip(linebuf, sizeof(linebuf), f_in) != NULL) {
-		if (memcmp(linebuf, "name", 4) == 0) {
-			p = &linebuf[4];
+		// isolate the attribute name
+		for (p = linebuf; (*p != 0) && !iswspace(*p); p++) ;
+		*p++ = 0;
+
+		if (linebuf[0] == 0) continue;
+
+		switch (LookupMnem(keyword_tab, linebuf))
+		{
+		case V_NAME:
 			while (isspace(*p)) p++;
 			strncpy0(vname, p, sizeof(vname));
-		} else if (memcmp(linebuf, "language", 8) == 0) {
+			break;
+		case V_LANGUAGE:
 			priority = DEFAULT_LANGUAGE_PRIORITY;
 			vlanguage[0] = 0;
 
-			sscanf(&linebuf[8], "%s %d", vlanguage, &priority);
+			sscanf(p, "%s %d", vlanguage, &priority);
 			len = strlen(vlanguage) + 2;
 			// check for space in languages[]
 			if (len < (sizeof(languages)-langix-1)) {
@@ -332,10 +315,15 @@ static espeak_VOICE *ReadVoiceFile(FILE *f_in, const char *fname, const char *le
 				langix += len;
 				n_languages++;
 			}
-		} else if (memcmp(linebuf, "gender", 6) == 0)
-			sscanf(&linebuf[6], "%s %d", vgender, &age);
-		else if (memcmp(linebuf, "variants", 8) == 0)
-			sscanf(&linebuf[8], "%d", &n_variants);
+			break;
+		case V_GENDER:
+			sscanf(p, "%s %d", vgender, &age);
+			if (is_language_file)
+				fprintf(stderr, "Error (%s): gender attribute specified on a language file\n", fname);
+			break;
+		case V_VARIANTS:
+			sscanf(p, "%d", &n_variants);
+		}
 	}
 	languages[langix++] = 0;
 
@@ -556,10 +544,15 @@ voice_t *LoadVoice(const char *vname, int control)
 			return NULL;
 	} else {
 		if (voicename[0] == 0)
-			strcpy(voicename, "en");
+			strcpy(voicename, ESPEAKNG_DEFAULT_VOICE);
 
 		sprintf(path_voices, "%s%cvoices%c", path_home, PATHSEP, PATHSEP);
 		sprintf(buf, "%s%s", path_voices, voicename); // look in the main voices directory
+
+		if (GetFileLength(buf) <= 0) {
+			sprintf(path_voices, "%s%clang%c", path_home, PATHSEP, PATHSEP);
+			sprintf(buf, "%s%s", path_voices, voicename); // look in the main languages directory
+		}
 	}
 
 	f_voice = fopen(buf, "r");
@@ -712,47 +705,57 @@ voice_t *LoadVoice(const char *vname, int control)
 			break;
 		case V_INTONATION: // intonation
 			sscanf(p, "%d %d", &option_tone_flags, &option_tone2);
-			if ((option_tone_flags & 0xff) != 0)
-				langopts->intonation_group = option_tone_flags & 0xff;
+			if ((option_tone_flags & 0xff) != 0) {
+				if (langopts)
+					langopts->intonation_group = option_tone_flags & 0xff;
+				else
+					fprintf(stderr, "Cannot set intonation: language not set, or is invalid.\n");
+			}
 			break;
 		case V_TUNES:
 			n = sscanf(p, "%s %s %s %s %s %s", names[0], names[1], names[2], names[3], names[4], names[5]);
-			langopts->intonation_group = 0;
-			for (ix = 0; ix < n; ix++) {
-				if (strcmp(names[ix], "NULL") == 0)
-					continue;
+			if (langopts) {
+				langopts->intonation_group = 0;
+				for (ix = 0; ix < n; ix++) {
+					if (strcmp(names[ix], "NULL") == 0)
+						continue;
 
-				if ((value = LookupTune(names[ix])) < 0)
-					fprintf(stderr, "Unknown tune '%s'\n", names[ix]);
-				else
-					langopts->tunes[ix] = value;
-			}
+					if ((value = LookupTune(names[ix])) < 0)
+						fprintf(stderr, "Unknown tune '%s'\n", names[ix]);
+					else
+						langopts->tunes[ix] = value;
+				}
+			} else
+				fprintf(stderr, "Cannot set tunes: language not set, or is invalid.\n");
 			break;
 		case V_DICTRULES: // conditional dictionary rules and list entries
 		case V_NUMBERS:
 		case V_STRESSOPT:
-			// expect a list of numbers
-			while (*p != 0) {
-				while (isspace(*p)) p++;
-				if ((n = atoi(p)) > 0) {
-					p++;
-					if (n < 32) {
-						if (key == V_DICTRULES)
-							conditional_rules |= (1 << n);
-						else if (key == V_NUMBERS)
-							langopts->numbers |= (1 << n);
-						else if (key == V_STRESSOPT)
-							langopts->stress_flags |= (1 << n);
-					} else {
-						if ((key == V_NUMBERS) && (n < 64))
-							langopts->numbers2 |= (1 << (n-32));
-						else
-							fprintf(stderr, "Bad option number %d\n", n);
+			if (langopts) {
+				// expect a list of numbers
+				while (*p != 0) {
+					while (isspace(*p)) p++;
+					if ((n = atoi(p)) > 0) {
+						p++;
+						if (n < 32) {
+							if (key == V_DICTRULES)
+								conditional_rules |= (1 << n);
+							else if (key == V_NUMBERS)
+								langopts->numbers |= (1 << n);
+							else if (key == V_STRESSOPT)
+								langopts->stress_flags |= (1 << n);
+						} else {
+							if ((key == V_NUMBERS) && (n < 64))
+								langopts->numbers2 |= (1 << (n-32));
+							else
+								fprintf(stderr, "Bad option number %d\n", n);
+						}
 					}
+					while (isalnum(*p)) p++;
 				}
-				while (isalnum(*p)) p++;
-			}
-			ProcessLanguageOptions(langopts);
+				ProcessLanguageOptions(langopts);
+			} else
+				fprintf(stderr, "Cannot set stressopt: language not set, or is invalid.\n");
 			break;
 		case V_REPLACE:
 			if (phonemes_set == 0) {
@@ -763,30 +766,31 @@ voice_t *LoadVoice(const char *vname, int control)
 			PhonemeReplacement(p);
 			break;
 		case V_WORDGAP: // words
-			sscanf(p, "%d %d", &langopts->word_gap, &langopts->vowel_pause);
+			if (langopts)
+				sscanf(p, "%d %d", &langopts->word_gap, &langopts->vowel_pause);
+			else
+				fprintf(stderr, "Cannot set wordgap: language not set, or is invalid.\n");
 			break;
 		case V_STRESSRULE:
-			sscanf(p, "%d %d %d %d", &langopts->stress_rule,
-			       &langopts->stress_flags,
-			       &langopts->unstressed_wd1,
-			       &langopts->unstressed_wd2);
-			break;
-		case V_CHARSET:
-			if ((sscanf(p, "%d", &value) == 1) && (value < N_CHARSETS)) {
-				if (new_translator != NULL)
-					new_translator->charset_a0 = charsets[value];
-				else
-					fprintf(stderr, "The charset attribute is specified before language.\n");
-			}
+			if (langopts)
+				sscanf(p, "%d %d %d %d", &langopts->stress_rule,
+				       &langopts->stress_flags,
+				       &langopts->unstressed_wd1,
+				       &langopts->unstressed_wd2);
+			else
+				fprintf(stderr, "Cannot set stressrule: language not set, or is invalid.\n");
 			break;
 		case V_OPTION:
-			value2 = 0;
-			if (((sscanf(p, "%s %d %d", option_name, &value, &value2) >= 2) && ((ix = LookupMnem(options_tab, option_name)) >= 0)) ||
-			    ((sscanf(p, "%d %d %d", &ix, &value, &value2) >= 2) && (ix < N_LOPTS))) {
-				langopts->param[ix] = value;
-				langopts->param2[ix] = value2;
+			if (langopts) {
+				value2 = 0;
+				if (((sscanf(p, "%s %d %d", option_name, &value, &value2) >= 2) && ((ix = LookupMnem(options_tab, option_name)) >= 0)) ||
+				    ((sscanf(p, "%d %d %d", &ix, &value, &value2) >= 2) && (ix < N_LOPTS))) {
+					langopts->param[ix] = value;
+					langopts->param2[ix] = value2;
+				} else
+					fprintf(stderr, "Bad voice option: %s %s\n", buf, p);
 			} else
-				fprintf(stderr, "Bad voice option: %s %s\n", buf, p);
+				fprintf(stderr, "Cannot set option: language not set, or is invalid.\n");
 			break;
 		case V_ECHO:
 			// echo.  suggest: 135mS  11%
@@ -844,9 +848,11 @@ voice_t *LoadVoice(const char *vname, int control)
 
 			name2[0] = 0;
 			sscanf(p, "%s %s %d", name1, name2, &srate);
-			if (LoadMbrolaTable(name1, name2, &srate) != ENS_OK)
-				fprintf(stderr, "mbrola voice not found\n");
-			voice->samplerate = srate;
+			espeak_ng_STATUS status = LoadMbrolaTable(name1, name2, &srate);
+			if (status != ENS_OK)
+				espeak_ng_PrintStatusCodeMessage(status, stderr, NULL);
+			else
+				voice->samplerate = srate;
 		}
 			break;
 		case V_KLATT:
@@ -861,37 +867,16 @@ voice_t *LoadVoice(const char *vname, int control)
 		case V_DICTMIN:
 			sscanf(p, "%d", &dict_min);
 			break;
-		case V_ALPHABET2:
-		{
-			ALPHABET *alphabet;
-			name1[0] = name2[0] = 0;
-			sscanf(p, "%s %s", name1, name2);
-
-			if (strcmp(name1, "latin") == 0)
-				strncpy0(langopts->ascii_language, name2, sizeof(langopts->ascii_language));
-			else if ((alphabet = AlphabetFromName(name1)) != 0) {
-				langopts->alt_alphabet = alphabet->offset;
-				langopts->alt_alphabet_lang = StringToWord2(name2);
-			} else
-				fprintf(stderr, "alphabet name '%s' not found\n", name1);
-		}
-			break;
-		case V_DICTDIALECT:
-			// specify a dialect to use for foreign words, eg, en-us for _^_EN
-			if (sscanf(p, "%s", name1) == 1) {
-				if ((ix = LookupMnem(dict_dialects, name1)) > 0)
-					langopts->dict_dialect |= (1 << ix);
-				else
-					fprintf(stderr, "dictdialect name '%s' not recognized\n", name1);
-			}
-			break;
 		case V_MAINTAINER:
 		case V_STATUS:
 			break;
 		default:
-			if ((key & 0xff00) == 0x100)
-				sscanf(p, "%d", &langopts->param[key &0xff]);
-			else
+			if ((key & 0xff00) == 0x100) {
+				if (langopts)
+					sscanf(p, "%d", &langopts->param[key &0xff]);
+				else
+					fprintf(stderr, "Cannot set voice attribute: language not set, or is invalid.\n");
+			} else
 				fprintf(stderr, "Bad voice attribute: %s\n", buf);
 			break;
 		}
@@ -1127,15 +1112,15 @@ static int ScoreVoice(espeak_VOICE *voice_spec, const char *spec_language, int s
 			score += 400;
 	}
 
-	if (((voice_spec->gender == 1) || (voice_spec->gender == 2)) &&
-	    ((voice->gender == 1) || (voice->gender == 2))) {
+	if (((voice_spec->gender == ENGENDER_MALE) || (voice_spec->gender == ENGENDER_FEMALE)) &&
+	    ((voice->gender == ENGENDER_MALE) || (voice->gender == ENGENDER_FEMALE))) {
 		if (voice_spec->gender == voice->gender)
 			score += 50;
 		else
 			score -= 50;
 	}
 
-	if ((voice_spec->age <= 12) && (voice->gender == 2) && (voice->age > 12))
+	if ((voice_spec->age <= 12) && (voice->gender == ENGENDER_FEMALE) && (voice->age > 12))
 		score += 5; // give some preference for non-child female voice if a child is requested
 
 	if (voice->age != 0) {
@@ -1190,7 +1175,7 @@ static int SetVoiceScores(espeak_VOICE *voice_select, espeak_VOICE **voices, int
 		}
 
 		sprintf(buf, "%s/voices/%s", path_home, language);
-		if (GetFileLength(buf) == -2) {
+		if (GetFileLength(buf) == -EISDIR) {
 			// A subdirectory name has been specified.  List all the voices in that subdirectory
 			language[lang_len++] = PATHSEP;
 			language[lang_len] = 0;
@@ -1309,7 +1294,7 @@ char const *SelectVoice(espeak_VOICE *voice_select, int *found)
 
 		if (voice_select2.name == NULL) {
 			if ((voice_select2.name = voice_select2.identifier) == NULL)
-				voice_select2.name = "default";
+				voice_select2.name = ESPEAKNG_DEFAULT_VOICE;
 		}
 
 		strncpy0(buf, voice_select2.name, sizeof(buf));
@@ -1319,7 +1304,7 @@ char const *SelectVoice(espeak_VOICE *voice_select, int *found)
 		if (vp != NULL) {
 			voice_select2.languages = &(vp->languages[1]);
 
-			if ((voice_select2.gender == 0) && (voice_select2.age == 0) && (voice_select2.variant == 0)) {
+			if ((voice_select2.gender == ENGENDER_UNKNOWN) && (voice_select2.age == 0) && (voice_select2.variant == 0)) {
 				if (variant_name[0] != 0) {
 					sprintf(voice_id, "%s+%s", vp->identifier, variant_name);
 					return voice_id;
@@ -1336,15 +1321,15 @@ char const *SelectVoice(espeak_VOICE *voice_select, int *found)
 	if (nv == 0) {
 		// no matching voice, choose the default
 		*found = 0;
-		if ((voices[0] = SelectVoiceByName(voices_list, "default")) != NULL)
+		if ((voices[0] = SelectVoiceByName(voices_list, ESPEAKNG_DEFAULT_VOICE)) != NULL)
 			nv = 1;
 	}
 
 	gender = 0;
-	if ((voice_select2.gender == 2) || ((voice_select2.age > 0) && (voice_select2.age < 13)))
-		gender = 2;
-	else if (voice_select2.gender == 1)
-		gender = 1;
+	if ((voice_select2.gender == ENGENDER_FEMALE) || ((voice_select2.age > 0) && (voice_select2.age < 13)))
+		gender = ENGENDER_FEMALE;
+	else if (voice_select2.gender == ENGENDER_MALE)
+		gender = ENGENDER_MALE;
 
 	#define AGE_OLD 60
 	if (voice_select2.age < AGE_OLD)
@@ -1361,7 +1346,7 @@ char const *SelectVoice(espeak_VOICE *voice_select, int *found)
 		// is the main voice the required gender?
 		skip = 0;
 
-		if ((gender != 0) && (vp->gender != gender))
+		if ((gender != ENGENDER_UNKNOWN) && (vp->gender != gender))
 			skip = 1;
 		if ((ix2 == 0) && aged && (vp->age < AGE_OLD))
 			skip = 1;
@@ -1405,7 +1390,7 @@ char const *SelectVoice(espeak_VOICE *voice_select, int *found)
 	return vp->identifier;
 }
 
-static void GetVoices(const char *path)
+static void GetVoices(const char *path, int len_path_voices, int is_language_file)
 {
 	FILE *f_voice;
 	espeak_VOICE *voice_data;
@@ -1430,16 +1415,16 @@ static void GetVoices(const char *path)
 			sprintf(fname, "%s%c%s", path, PATHSEP, FindFileData.cFileName);
 			ftype = GetFileLength(fname);
 
-			if (ftype == -2) {
-				// a sub-sirectory
-				GetVoices(fname);
+			if (ftype == -EISDIR) {
+				// a sub-directory
+				GetVoices(fname, len_path_voices, is_language_file);
 			} else if (ftype > 0) {
-				// a regular line, add it to the voices list
+				// a regular file, add it to the voices list
 				if ((f_voice = fopen(fname, "r")) == NULL)
 					continue;
 
 				// pass voice file name within the voices directory
-				voice_data = ReadVoiceFile(f_voice, fname+len_path_voices, FindFileData.cFileName);
+				voice_data = ReadVoiceFile(f_voice, fname+len_path_voices, is_language_file);
 				fclose(f_voice);
 
 				if (voice_data != NULL)
@@ -1466,16 +1451,16 @@ static void GetVoices(const char *path)
 
 		ftype = GetFileLength(fname);
 
-		if (ftype == -2) {
-			// a sub-sirectory
-			GetVoices(fname);
+		if (ftype == -EISDIR) {
+			// a sub-directory
+			GetVoices(fname, len_path_voices, is_language_file);
 		} else if (ftype > 0) {
-			// a regular line, add it to the voices list
+			// a regular file, add it to the voices list
 			if ((f_voice = fopen(fname, "r")) == NULL)
 				continue;
 
 			// pass voice file name within the voices directory
-			voice_data = ReadVoiceFile(f_voice, fname+len_path_voices, ent->d_name);
+			voice_data = ReadVoiceFile(f_voice, fname+len_path_voices, is_language_file);
 			fclose(f_voice);
 
 			if (voice_data != NULL)
@@ -1583,9 +1568,11 @@ ESPEAK_API const espeak_VOICE **espeak_ListVoices(espeak_VOICE *voice_spec)
 	FreeVoiceList();
 
 	sprintf(path_voices, "%s%cvoices", path_home, PATHSEP);
-	len_path_voices = strlen(path_voices)+1;
+	GetVoices(path_voices, strlen(path_voices)+1, 0);
+
+	sprintf(path_voices, "%s%clang", path_home, PATHSEP);
+	GetVoices(path_voices, strlen(path_voices)+1, 1);
 
-	GetVoices(path_voices);
 	voices_list[n_voices_list] = NULL; // voices list terminator
 	espeak_VOICE **new_voices = (espeak_VOICE **)realloc(voices, sizeof(espeak_VOICE *)*(n_voices_list+1));
 	if (new_voices == NULL)
